#! /usr/bin/perl

################################################################
###                                                          ###
### SCaFoS                                                   ###
###                                                          ###
################################################################
### Version 4.42 du 2006/10/11                               ###
################################################################


 use strict;
 use warnings;

 use Cwd;

 use lib "$ENV{SCAFOS}/scafosmod";
 use Var124;
 use Lib124;
 use Gui124;
 use Gub124;
 use Run124;


################################################################


 my $Header =    "       SSSSS    CCCCC          FFFFFFF       SSSSS\n";
    $Header .=   "      SS   SS  CC   CC         FF           SS   SS\n";
    $Header .=   "       SSS    CC               FF            SSS         11     222       444\n";
    $Header .=   "        SSS   CC        AAAAA  FFFFF  OOOO    SSS       111    22 22     4444\n";
    $Header .=   "         SSS  CC       AA  AA  FF    OO  OO    SSS     1111      22     44 44\n";
    $Header .=   "      SS   SS  CC   CC AA  AA  FF    OO  OO SS   SS      11     22     4444444\n";
    $Header .=   "       SSSSS    CCCCC   AAAA A FF     OOOO   SSSSS       11 ## 22222 ##    44\n";
 
 my $L;

 
################################################################


 #-- verifie OS ------------------------------------------------
 if ( $^O eq 'MSWin32' ) 
    { $SepDir = '\\'; }
 else
    # linux-like dont Mac OS X
    { $SepDir = '/'; }    

 #-- affichage de l'aide en mode texte -------------------------
 if ( $#ARGV == 0 ) {
    if ( $ARGV[0] =~ m/help|^h|\?/i ) { 
        PrintHeader();
        Usage(''); 
        }
    }
 
 #-- initialisation des variables ------------------------------
 InitVar();
 
 #-- lance mode graphique --------------------------------------
 if ( $#ARGV == -1 ) {
    $FlagIHM = 1;
    GraphicalUse();
    }
 
 #-- lance mode texte ------------------------------------------
 else {
    $FlagIHM = 0;
    ParamIn();
    LoadCEF();
    }
    
 exit(0);

 
### fin du programme                                         ###
################################################################


################################################################
### INITIALISATION DU PROGRAMME ################################

# NOM : InitVar
# DESCRIPTION : Initialisation des variables d'entree
# PARAMETRES : so
# RETOUR : so
sub InitVar {
    $CefRep = '';
    $PuzRep = '';
    $NameCef = '';
    $FileRep = '';
    $SpecFile = '';
    $NameFile = '';
    $TauxMin = 0;
    $FlagCat = 0;
    $FlagFile = 0;
    $FlagAll = 0;
    $FlagOnly = 0;
    $FlagChim = 0;
    $PctAve = 0.25;
    $FlagNoPuz = 0;
    $FlagGamma = 0;
    $FlagDef = 0;    
    $FlagDefAuto = 0;    
    $MissPos = 10;
    $FlagColor = 0;
    $NbMisGrp = 25;
    $MisGenRep = '';
    $FlagMisGrp = 0;
    $MisSitRep = '';
    $FlagMisSit = 0;
    $PctMisSit = 20;
    $PasMisSit = 5;
    $FlagFasta = 0;
    $FlagMust = 0;
    $FlagPHYLIP = 0;
    $FlagNexus = 0;
    $FlagMB = 0;
    $FlagPAUP = 0;
    $FlagOther = 0;
    $MinLen = 10;
    $FlagVerb = 0;
    $FlagAlphabet = 0;
    $FlagLength = 0;
    $FlagAutoCat = 0;
    $FlagCmp = 0;
    $FlagRemoveAll = 0;
}


################################################################
### EXECUTION EN COMMANDE EN LIGNE #############################


# NOM : PrintHeader 
# DESCRIPTION : affiche l'entete du programme
# PARAMETRES : so
# RETOUR : so
sub PrintHeader {
    #-- affichage de l'ente ---
    $L = "\n\n================================================================================\n\n";
    $L .= $Header;
    $L .= "\n\n                 SELECTION, CONCATENATION and FUSION of SEQUENCES\n";
    $L .= "                                   ($DateVers)\n";
    $L .= "\n================================================================================";
    $L .= "\n      UNIVERSITE DE MONTREAL                       CENTRE ROBERT CEDERGREN";
    $L .= "\n================================================================================\n\n\n";
    print $L;
}


# NOM : ParamIn 
# DESCRIPTION : gestion des parametres d entree
# PARAMETRES : so
# RETOUR : so
sub ParamIn {   
    my $buf;
    $CefRep = $SepDir;
    
    ### Affichage de l'entete ###
    PrintHeader();

    ### Affichage de l'aide ###  
    if ( $#ARGV == -1 ) {
        $buf = "   SELECTION OPTIONS ARE NECESSARY ";
        Usage($buf); 
        }
    if ( $ARGV[0] =~ m/help|^h|\?/i ) { Usage(''); }

    ### Gestion des parametres d'entree ###    
    $FlagFasta = 1;   # Sortie au format FASTA
    $FlagNexus = 1;   # Sortie au format NEXUX
    $FlagMB = 1;      # Sortie au format NEXUX avec commandes pour MrBayes
    #-- recuperation saisie ------------------------------------
    foreach ( @ARGV ) {
        if ( $_ =~ m/^in=(.*)/i )
            { $FileRep = $1; }
        elsif ( $_ =~ m/^out=(.*)/i ) {
            $CefRep = $1;
            $L = rindex($1, $SepDir);
            if ( ($L == -1) or ($L < length($CefRep)-1) ) { $CefRep .= $SepDir; }
            }
        elsif ( $_ =~ m/^frq=(.*)/i )
            { $TauxMin = $1; }
        elsif ( $_ =~ m/^otu=(.*)/i )
            { $SpecFile = $1; }
        elsif ( $_ =~ m/^puz=(.*)/i ) {
            if ( $1 =~ m/no|n/i ) { $FlagNoPuz = 1; }
            }
        elsif ( $_ =~ m/^gamma=(.*)/i ) {
            if ( $1 =~ m/yes|y/i ) { $FlagGamma = 1; }
            }
        elsif ( $_ =~ m/^t=(.*)/i )
            { $PctAve = $1 / 100; }
        elsif ( $_ =~ m/^o=(.*)/i ) {
            $L = $1;
            if ( $L =~ m/f/i )
                { $FlagFile = 1; }
            elsif ( $L =~ m/a/i )
                { $FlagAll = 1; }
            elsif ( $L =~ m/o/i )
                { $FlagOnly = 1; }
            else {
                if ( $L =~ m/v/i ) { $FlagVerb = 1; }
                if ( $L =~ m/c/i ) { $FlagChim = 1; $FlagCat = 1; }
                if ( $L =~ m/s/i ) { $FlagMisSit = 1; $FlagCat = 1; }
                if ( $L =~ m/g/i ) { $FlagMisGrp = 1; $FlagCat = 1; }
                if ( $L =~ m/l/i ) { $FlagLength = 1; $FlagCat = 1; }
                if ( $L =~ m/r/i ) { $FlagRemoveAll = 1; $FlagCat = 1; }
                }
            }
        elsif ( $_ =~ m/^name=(.*)/i )
            { $NameFile = $1; }
        elsif ( $_ =~ m/^def=(.*)/i ) {
            if ( $1 =~ m/yes|y/i ) { $FlagDef = 1; }
            }
        elsif ( $_ =~ m/^auto=(.*)/i ) {
            if ( $1 =~ m/yes|y/i ) { $FlagDefAuto = 1; }
            }
        elsif ( $_ =~ m/^m=(.*)/i )
            { $MissPos = int($1); }
        elsif ( $_ =~ m/^color=yes/i )
            { $FlagColor = 1; }
        elsif ( $_ =~ m/^s=(.*)/i )
            { $PctMisSit = int($1); }
        elsif ( $_ =~ m/^p=(.*)/i )
            { $PasMisSit = int($1); }
        elsif ( $_ =~ m/^g=(.*)/i )
            { $NbMisGrp = int($1); }
        elsif ( $_ =~ m/^l=(.*)/i )
            { $MinLen = int($1); }
        elsif ( $_ =~ m/^cat=yes/i )
            { $FlagAutoCat = 1; }
        elsif ( $_ =~ m/^cmp=yes/i )
            { $FlagCmp = 1; }
        elsif ( $_ =~ m/^format=(.*)/i ) {
            $FlagFasta = 0;
            $FlagNexus = 0;
            $FlagMB = 0;
            $L = $1;
            if ( $L =~ m/m/i )
                { $FlagMust = 1; }
            if ( $L =~ m/f/i )
                { $FlagFasta = 1; }
            if ( $L =~ m/p/i )
                { $FlagPHYLIP = 1; }
            if ( $L =~ m/n/i )
                { $FlagNexus = 1; }
            if ( $L =~ m/b/i )
                { $FlagMB = 1; $FlagNexus = 1; }
            if ( $L =~ m/a/i )
                { $FlagPAUP = 1; $FlagNexus = 1; }
            }
#!# drapeau pour test de SCaFoS
elsif ( $_ =~ m/^test=(.*)/ ) { $FlagTest = $1; }
#!#
        else {
            $buf = "   \'$_\' IS AN INCORRECT OPTION";
            Usage($buf);
            }
        }
    
#print "FlagTest=$FlagTest\n";
#print "TauxMin=$TauxMin\tPctAve=$PctAve\tFlagCat=$FlagCat\tFlagChim=$FlagChim\n";
#print "FlagNoPuz=$FlagNoPuz\tFlagDef=$FlagDef\tFlagDefAuto=$FlagDefAuto\t\n";
        
    #-- verification saisie ------------------------------------
    if ( $FileRep eq '' ) {
        $buf = "  IN DIRECTORY \'in=\' IS NECESSARY";
        Usage($buf);
        }
    if ( $FlagFile == 0 and $FlagAll == 0 and $FlagOnly == 0 and $FlagCat == 0 and $SpecFile ne '' ) { $FlagCat = 1; }
    if ( ( ($FlagFile == 1) or ($FlagAll == 1) or ($FlagOnly == 1) or ($FlagCat == 1)) and ($SpecFile eq '') ) {
        $buf = "   WITH YOUR OPTIONS FILE OF SPECIES \'spe=\' IS NECESSARY";
        Usage($buf);
        }
    if ( ($FlagDefAuto == 1) and ($FlagDef == 0) ) {
        $buf = "   AUTO OPTION \'auto=yes\' MUST BE USED WITH DEF OPTION \'def=yes\'";
        Usage($buf);
        }
    if ( ($FlagRemoveAll == 1) and ($FlagDef == 1) ) {
        $buf = "   AUTO OPTION \'auto=yes\' is INCOMPATIBLE WITH REMOVAL OPTION \'o=r\'";
        Usage($buf);
        } 
    # test si le repertoire de sortie existe deja ---
    my $flag = 0;
    while ( $flag == 0 )
        { ($flag, $CefRep) = TestOut($CefRep); }
    $NameCef = Get_NameCef($CefRep);
    $PuzRep = $CefRep.'puz'.$SepDir;
    $MisGenRep = $CefRep.'misgen'.$SepDir;
    $MisSitRep = $CefRep.'missit'.$SepDir;
    # test si % donnees manquantes compris entre 0 et 50 ---
    if ( ($MissPos < 0) or ($MissPos > 50) ) {
        $buf = "   PERCENT OF MISSING SITES FOR COMPLETE SEQUENCE \'m=\' IS AN INTEGER VALUE BETWEEN 0 AND 50";
        Usage($buf);
        }
    if ( ($PctMisSit == -1) and ($FlagMisSit == 1) ) { $PctMisSit = 20; }
    if ( ($PctMisSit < 0) or ($PctMisSit > 100) ) {
        $buf = "   MAX PERCENT OF MISSING SITES IN SUBDIRECTORIES \'s=\' IS AN INTEGER VALUE BETWEEN 0 AND 100";
        Usage($buf);
        }
    if ( $PasMisSit == 0 ) {
        $buf = "   STEP OF MISSING SITES \'p=\' IS AN INTEGER VALUE BETWEEN 1 AND MAX PERCENT OF MISSING SITES \'s=\'";
        Usage($buf);
        }
    else {
        if ( ($PctMisSit % $PasMisSit) != 0 ) {
            $buf = "   MAX PERCENT OF MISSING SITES \'s=\' MUST BE DIVIDED BY STEP OF MISSING SITES \'p=\'";
            Usage($buf);
            }
        }
    if ( ($MinLen < 0) or ($MinLen > 100) ) {
        $buf = "   PERCENT OF SEQUENCE LENGHT \'l=\' IS AN INTEGER VALUE BETWEEN 0 AND 100";
        Usage($buf);
        }
    if ( $FlagMust == 0 and $FlagFasta == 0 and $FlagPHYLIP == 0 and $FlagNexus == 0 and $FlagMB == 0 and $FlagPAUP == 0 ) {
        $buf = "   OUTPUT FORMAT FILE is NOT CORRECT";
        Usage($buf);
        }
}


# NOM : TestOut 
# DESCRIPTION : test si le repertoire de sortie existe deja
# PARAMETRES : nom du repertoire de sortie
# RETOUR : 1 -> repertoire accepte
#          0 -> nouvelle saisie
sub TestOut {
    my ($Out) = @_;
    
    if ( UnconvenientOut($Out) == -1 ) { exit(-1); }
        
    # repertoire existant
    my $Rep = getcwd();
    if ( chdir $Out ) {
        chdir $Rep;
        print "\n $Out already exists. Do you want erase it ? y(es) / n(o) / c(ancel) > ";
        $L = <STDIN>;
        print "\n";
        $L =~ s/\s+//g;
        $L =~ s/A-Z/a-z/g;
        if  ( $L =~ m/yes|y/i ) {
            if ( $^O eq 'MSWin32' )  { system("rmdir /S /Q $Out"); }
            else { system("rm -fR $Out"); }
            return (1, $CefRep);
            }
        elsif ( $L =~ m/no|n/i ) {
            print " name of new output > ";
            chomp($L = <STDIN>);
            if ( UnconvenientOut($L.$SepDir) == -1 ) { exit(-1); }
            print "\n";
            return (0, $L.$SepDir);
            }
        elsif ( $L =~ m/cancel|c/i ) {
            die "\n\n=======================================================================\n\n                              END OF SCaFos\n\n=======================================================================\n\n";   
            }
        }
    else
        { return (1, $Out); }
}

            
# NOM : Usage 
# DESCRIPTION : affiche le mode d'emploi du programme et quitte
# PARAMETRES : so
# RETOUR : so
sub Usage {
    my ($Err) = @_;
    my $buf;
    
    $buf  = " USAGE of SCaFoS :\n";
    $buf .= " ---------------\n";
    $buf .= "\n DESCRIPTION\n";
    $buf .= "    Concatenation and extraction of sequences from aligned sequences files\n";
    $buf .= "    version: $VERSION ($DateVers)\n";
    $buf .= "\n SYNOPSIS\n";
    $buf .= "    to make file.otu: (usage 1)\n";
    $buf .= "        scafos in=<dir1> out=<dir2>\n";
    $buf .= "               [frq=<freq>] [name=<file.nom>] [l=<minlen>] [cat=yes]\n";
    $buf .= "\n    to select files with choosen OTUs: (usage 2)\n";
    $buf .= "        scafos in=<dir1> out=<dir2> otu=<file>.otu o=[ofav] [l=<minlen>]\n";
    $buf .= "\n    to construct datasets: (usage 3)\n";
    $buf .= "        scafos in=<dir1> out=<dir2> otu=<file>.otu\n";
    $buf .= "               [gamma=yes] [t=<threshold>] [puz=no]\n";
    $buf .= "               [o=gsclrv g=<number> s=<number> p=<step>]\n";
    $buf .= "               [def=yes auto=yes] [m=<missing>] [l=<minlen>]\n";
    $buf .= "               [format=fpmnba] [color=yes] [cmp=yes]\n";
    $buf .= "\n    to use in a graphical mode: (usage 0)\n";
    $buf .= "        scafos\n";
    $buf .= "\n    to quit this help: q\n";

    $L = '';
    while ( !($L =~ m/\d/) ) {
        print $buf . "\n type the usage number to view detailed options (or 0 to cancel)> ";
        $L = <STDIN>;
        print "\n";
        $L =~ s/\s+//g;
        if  ($L eq 'q') { exit(0); }
        $L = int($L);
        }
    if  ( $L eq '0') {
        exit(0);
        }
    $buf = "\n OPTIONS";
    if ( $L eq '1' ) {
        $buf .= " to make file.otu\n";
        $buf .= "    dir1      directory of input files .ali\n";
        $buf .= "    dir2      directory of output\n";
        $buf .= "    freq      minimum frequency of species in files (default=0)\n";
        $buf .= "    file.nom  list of species with corresponding taxa\n";
        $buf .= "    minlen    minimal percent of sequence length to keep a sequence (default=10)\n";
        $buf .= "    cat=yes   to automatically concatenate files\n";
        }
    elsif ( $L eq '2' ) {
        $buf .= " to select files with choosen OTUs\n";
        $buf .= "    dir1      directory of input files .ali\n";
        $buf .= "    dir2      directory of output\n";
        $buf .= "    file.otu  list of OTUs used for concatenation\n";
        $buf .= "    o         write files ONLY with SELECTED SPECIES in dir2\n";
        $buf .= "    f         copy files AT LEAST with ONE SELECTED SPECIES in dir2\n";
        $buf .= "    a         copy files with ALL OTUs in dir2\n";
        $buf .= "    v         verbose\n";
        $buf .= "    minlen    minimal percent of sequence length to keep a sequence (default=10)\n";
        }
    elsif ( $L eq '3' ) {
        $buf .= " to construct datasets\n";
        $buf .= "    dir1      directory of input files .ali\n";
        $buf .= "    dir2      directory of output\n";
        $buf .= "    file.otu  list of OTUs used for concatenation\n";
        $buf .= "    threshold threshold used to compare distances from TREE-PUZZLE (default=25)\n";
        $buf .= "    gamma=yes use of 4 gamma categories to calculate evolutionary distances\n";
        $buf .= "    g         make subdirectories of files containing less than 'number' missing\n";
        $buf .= "              OTUs (default=number of OTUs)\n";
        $buf .= "    s         make subdirectories of files containing less than 'number' % of\n";
        $buf .= "              missing sites (default=20)\n";
        $buf .= "    c         make chimera with partial sequences in an OTU\n";
        $buf .= "    l         add caracters number in concatenated sequence after sequence name\n";
        $buf .= "    r         remove completely a file when at least one OTU does not respect\n";
        $buf .= "              the threshold (default=no removal action)\n";
        $buf .= "              ! no compatible with def option\n";
        $buf .= "    v         verbose\n";
        $buf .= "    number    write list of files according to :\n";
        $buf .= "                 'number' missing OTUs in .misgp (default=number of OTUs) if o=g\n";
        $buf .= "                 'number' percent of missing sites in file (default=0) if o=s\n";
        $buf .= "    p         step of percent of missing sites to create sub-directories (default=5)\n";
        $buf .= "    puz=no    doesn't use PUZZLE to determine best sequence (default=use of TREE-PUZZLE)\n";
        $buf .= "              ! no compatible with def option\n";
        $buf .= "    def=yes   use of input files *.def for selected sequences (default=sequences\n";
        $buf .= "              select by SCaFos computation)\n";
        $buf .= "              ! no compatible with removal option (o=r)\n";
        $buf .= "              ! no compatible with puz option\n";
        $buf .= "    auto=yes  use SCaFos computation and don't ask user if more one sequences is\n";
        $buf .= "              possible (default=no)\n";
        $buf .= "              ! must be used only with def option\n";
        $buf .= "    missing   percent of missing sites to use sequence as a complete sequence [0:50]\n";
        $buf .= "              (default=10)\n";
        $buf .= "    minlen    minimal percent of sequence length to keep a sequence (default=10)\n";
        $buf .= "    f         output files in FASTA format\n";
        $buf .= "    p         output files in PHYLIP format\n";
        $buf .= "    m         output files in MUST format\n";
        $buf .= "    n         output files in NEXUS format\n";
        $buf .= "    b         output files in NEXUS format with commands for MrBAYES\n";
        $buf .= "    a         output files in NEXUS format with commands for PAUP\n";
        $buf .= "    color=yes modify colors in <tree>.ps files (default=no)\n";
        $buf .= "              green->chimera     blue->selected sequence     grey->rejected sequence\n";
        $buf .= "    cmp=yes   computing of composition in amino acids or nucleotids\n";
        }

    $buf .= "\n\n################################################################################\n\n";
    if ( length($Err) > 0 ) {
        $buf .= $Err;
        $buf .= "\n\n################################################################################\n\n";
        }
    die $buf;
}


### fin du fichier                                           ###
################################################################
