
 package Cat124;

################################################################
###                                                          ###
### SCaFoS                                                   ###
### Concatenation des sequences                              ###
###                                                          ###
################################################################
### Version 4.40 du 2006/09/01                           BPH ###
################################################################

 use strict;
 use warnings;
 use Cwd;
 
 use Var124;
 use Lib124;
 use Puz124;
 use Def124;
 use Out124;
 use Gub124;

 use vars qw(@ISA @EXPORT);
 require Exporter;
 @ISA = qw(Exporter);
 @EXPORT = qw/ LogHeader AppelConcat /;

 
################################################################


 my $L;
 my $bAff = '';       # texte pour affichage en mode graphique


### Ecriture de l entete #######################################
sub LogHeader {
    $L = "\n\n=======================================================================\n";
    $L .= "\n Directory : $FileRep\n";
    $L .= " Number of genes for studies : $NbFileAli\n";
    if ( $#ARGV >= 2 ) {
        if ( length($SpecFile) > 0 ) {
            $L .= " Concat sequences from file \'$SpecFile\'\n";
            $L .= " Number of species retained: $nbSpe / $NbEspTot";
            $L .= "\n\n=======================================================================\n\n";
            if ( $FlagIHM == 1 )
                { ViewDetails($L, 0); }
            else
                { print $L; }
            }
        else
            { $L .= " Minimal species\' frequency in a file : $TauxMin%\n"; }
        }
    else {
        $L .= " Count of frequency for each species";
        $L .= "\n\n=======================================================================\n\n\n";
        if ( $FlagIHM == 1 )
            { ViewDetails($L, 0); }
        else
            { print $L; }
        }
    push (@Resume, $L);
}


### Concatenation ##############################################
sub AppelConcat {
    my $bResult = '';
    my $i;
    
    %hConcat = ();
    
    if ( $TauxMin eq '' ) { $TauxMin = 0; }
    #-- Lecture fichier SPE ------------------------------------
    if ( !open(FIC, $SpecFile) )
        { AffError("Can\'t open for reading the file \'$SpecFile\' !"); }
    $nbSpe = 0;
    while ( <FIC> ) {
        $L = $_;
        $L =~ s/^\s+//;
        if ( !($L =~ m/^#(.*)/) and (length($L) > 1) ) { $nbSpe++; }
        }
    close FIC;

    #-- Gestion groupes manquants ------------------------------
    if ( $NbMisGrp == -1 ) { $NbMisGrp = $nbSpe; }
    if ( $FlagMisGrp == 1 ) {    
        for ( $i = 0; $i <= $NbMisGrp; $i++ )
            { CreateDir($MisGenRep."misgen_$i"); }
        }
    if ( $FlagMisSit == 1 ) {    
        for ( $i = 0; $i <= $PctMisSit; $i = $i+$PasMisSit )
            { CreateDir($MisSitRep."missit_$i"); }
        }

    LogHeader();
    
    #-- concatenation des sequences ----------------------------
    %hConcat = ConcatSeqFromFile();
#     if ( $^O eq 'MSWin32' ) { 
#         system("del /s /q $PuzRep\*");
# #        system("rmdir /s /q $PuzRep");
#         }
# #    else { system("rm -fR $PuzRep"); }
#     rmdir($PuzRep);
    RemoveDir($PuzRep);
    
    #-- Affichage des resultats ---
    if ( $FlagNoPuz == 0 )
        { $bResult .= "\n\n   Details of the concatenation in files \'$NameCef.dist.log\' and \'$NameCef.seq4otu\'"; }
    else
        { $bResult .= "\n\n   Details of the concatenation in file \'$NameCef.seq4otu\'"; }
    $L .= "\n\n   List of files according number of OTUs is in \'$NameCef.misotu\'";
    $L .= "\n\n   Details of missing data in files \'$NameCef.stat\'";
    if ( $FlagChim == 1 )
        { $bResult .= "\n\n   Details of making of chimeras in file \'$NameCef.chim.log\'"; }
    if ( $FlagMisSit == 1 )
        { $bResult .= "\n\n   Files extracted from .ali are in directories \'$MisGenRep\' and \'$MisSitRep\'"; }
    else
        { $bResult .= "\n\n   Files extracted from .ali are in directory \'$MisGenRep\'"; }
    if ( $FlagDef == 1 )
        { $bResult .= "\n\n   New lists of default sequences in files \'$CefRep\*.def\'"; }
    $bResult .= "\n\n=======================================================================\n\n\n";
    
    #-- Pas de concatenation ---
    if ( scalar(keys(%hConcat)) == 0 ) {
        $L = "\n\n\n=======================================================================";
        $L .= "\n\n RESULTS";
        $L .= "\n\n   Problem during concatenation";
        $bResult = $L . $bResult;
        if ( $FlagIHM == 1 ) { ViewDetails($bResult, 0); }
        else { print $bResult; }
        return();
        }
    #-- Verifie OTUs vides ---
    my $buf = '';
    foreach $L (sort keys(%hConcat)) {
        if ( CountPosMis($hConcat{$L}) == length($hConcat{$L}) )
            { $buf .= "    $L\n"; }
        }
    if ( length($buf) > 0 ) {
        if ( $FlagIHM == 0 ) 
            { print "\n  List of empty OTUs:\n$buf\n\n"; }
        elsif ( $FlagVerb == 1 )
            { ViewDetails("\n List of empty OTUs:\n$buf\n\n", 0); }
        }
    #-- Ecriture des sequences concatenees ---------------------
    WriteConcat($bResult);
}


### Lecture du fichier .otu ####################################
sub ReadSpe {    
    my $Motif;
    my $Nom;
    my ($i, $pos, $deb,  $rg);
    my ($L, $L2, $L3);
    my $Id10 = "";
    my $bufId = "";     # liste des identifiants uniques
    my %hGrpId;         # hachage ( nom sequence => max index )
    my $ListEsp = "<<<<<";   # liste des especes retenues par l utilisateur
    
    if ( !open(SPE, $SpecFile) ) { AffError("Can't open for reading the file \'$SpecFile\' !"); }

    $bAff = " READING OF \'$SpecFile\' ";
    if ( $FlagIHM == 1 ) { ViewDetails($bAff, 0); }
    else { print $bAff; }
    while ( $L = <SPE> ) {
        $Nom = "";
        $L2 = "";
        
        $L =~ s/^\s+//;
        if ( $L =~ m/^#/ ) { next; }
        
        #** separe nom du groupe des especes a prendre en compte pour la concatenation **
        if ( $L =~ m/(.*):(.*)/) {
            $Nom = $1;  # nom d espece utilise pour la concatenation = groupe
            $L2 = $2;   # liste des especes du groupe
            $Nom =~ s/^\s+//;
            $Nom =~ s/\s+$//;
            $L2 =~ s/,\s*/\|/g;
            $L2 =~ s/,/\|/g;
            $L2 .= "|";
            
            #-- construit motif de recherche pour .ali --
            $Motif = "<<<<<";
            $i = 0;
            pos($L2) = 0;
            ($pos,$deb) = (0,0);
            while ( ($pos = index($L2, "\|", $deb)) > -1 ) {
                $L3 = substr($L2, $deb, $pos - $deb);
                $L3 =~ s/\s?\(.*\)//;
                $L3 =~ s/^\s+//;
                $L3 =~ s/\s+$//;
                $Motif .= $L3."<<<<<";
                $hGrpSpe{$Nom}{$L3} = $i++;
                $ListEsp .= $L3."<<<<<";
                $deb = $pos + 1;
                }
            $hGrp{$Nom} = $Motif;
            
            #-- cree identifiants sur 10 carcateres --
            $Id10 = substr($Nom, 0, 10);
            if ( $bufId =~ m/$Id10/i ) {
                ++($hGrpId{$Id10});
                if ( $hGrpId{$Id10} < 10 ) { $Id10 = substr($Id10, 0, 9) . $hGrpId{$Id10}; }
                else { $Id10 = substr($Id10, 0, 8) . $hGrpId{$Id10}; }
                }
            else {
                $hGrpId{$Id10} = 0;
                $bufId .= ($Id10 . " - ");
                }
            @{$hIdentGrp{$Nom}}[0] = $Id10;
            @{$hIdentGrp{$Nom}}[1] = ++$rg;
            }
        }
    
    close SPE;
    return ($ListEsp);
}


### Concatenation des sequences depuis le fichier .otu #########
sub ConcatSeqFromFile {
    my $L;
    my $ListEsp;        # liste des especes retenues par l'utilisateur
    my $rg = 0;         # rang de l'espece dans le fichier .otu
    my $KeyFile = "";
    my $NbGrpAbs;       # nombre de groupes absents d'un fichier 
    my $LgFile;         # longueur max sequences dans un fichier
    my $bLOG2;

    #** Lecture du fichier SPE *********************************
    if ( $FlagIHM == 1 ) { ViewCurrent('Reading of species file', 'darkblue', $Current ); }
    $ListEsp = ReadSpe();
    
    #** Appel de puzzle ****************************************
    SetPuzzleDist($ListEsp);
        
    #** Gestion fichier log ************************************
    if ( $FlagNoPuz == 0 ) {
        $L = $NameCef.".outdist";
        open(LOG2,"> $L") || die "\nCan\'t open for writing the file \'$L\' !\n\n";
        if ( !open(LOG2,"> $L") ) { AffError("Can't open for writing file \'$L\' !"); }
        
        print LOG2 "-----------------------------------------------------------------------\n";
        print LOG2 " In directory : $FileRep\n";
        print LOG2 " Out directory : $CefRep\n";
        print LOG2 " File of species : $SpecFile\n";
        print LOG2 " ".scalar(localtime(time()))." \n";
        print LOG2 "-----------------------------------------------------------------------\n";
        print LOG2 "\n\n\n=======================================================================\n";
        print LOG2 "\n\n SEQUENCES OUT OF AVERAGE OF DISTANCES\n\n";
        print LOG2 " Percent of average of distances authorized for \n";
        print LOG2 " the maximum distance within an OTU : ".100*$PctAve."%\n";
    }
    
    #** Concatenation des sequences ****************************
    if ( $FlagVerb == 1 ) {
        $bAff = "\n\n=======================================================================\n";
        $bAff .= "\n CONCATENATION\n";
        }
    else
        { $bAff = "\n CONCATENATION\n"; }
    if ( $FlagIHM == 1 ) { ViewDetails("\n$bAff", 0); }
    else { print $bAff; }

    #** sauvegarde sequence conservee pour chaque groupe *******
    $L = $NameCef.".seq4otu";
    if ( !open(LOG4,"> $L") ) { AffError("Can't open for writing file \'$L\' !"); }
    
    #** sauvegarde longueur sequence et nombre de sequences pour chaque fichier    
    $L = $NameCef.".len";
    if ( !open(LEN,"> $L") ) { AffError("Can't open for writing file \'$L\' !"); }
# 4.40    print LEN "#File\tLength\tNo seq\n";
    print LEN "#File\tBegin\tEnd\t[Length\tNo OTUs]\n";

    my $F;
    my ($Grp, $Seq, $buf, $Name);
    my @Fic;
    @Fic = sort keys %Fichiers;
    my @Grp;
    @Grp = sort keys %hGrp;
    
    for ( @Grp ) { 
        $hStatGrp{$_}[0] = 0;
        $hStatGrp{$_}[1] = 0;
        $hStatGrp{$_}[2] = 0;
        $hConcat{$_} = '';
        }

    #** boucle sur les fichiers .ali ***************************
    my ($bHeader, $bFooter, $bLeafIn, $bLeafOut, $fColor);
    my ($nbSeq, $Pos, $iSeq, $n, $flag, $MaxBlanc);
    my ($bLOG4, $bEXT, $bRemFile, %hConcatTmp, @lStatFicTmp, %hStatGrpTmp);
    $bRemFile = "";
    $fColor = 0;
    if ( $FlagIHM == 1 ) { ViewCurrent('Reading of aligned file', 'darkblue', $Current ); }
    
    my $Deb = 1;
    for $F ( @Fic ) {
        if ( $FlagIHM == 1 ) { ViewDetails("\n   === $F ===", 1); }
        elsif ( $FlagVerb == 1 ) { print "\n   === $F ==="; }
        
        #** initialisation des variables ***
        ($Grp, $Seq, $buf, $Name) = ("", "", "", "");
        ($nbSeq, $Pos) = (0, 0);
        $iSeq = 0;             # nb sequences selectionnees dans le fichier
        $NbGrpAbs = 0;         # nb d'OTUs absentes dans le fichier
        $bLOG2 = '';
        $bLOG4 = "\n=== File: $F ===\n";
        $bEXT = "";
        
        $F =~ m/(.*)\.ali$/i;
        $KeyFile = $1;
        $LgFile = $FicLen{$F};
        $MaxBlanc = sprintf("%.0f", $LgFile * (1-($MinLen / 100)));
        $bEXT = "#Number of positions: $LgFile\n#\n";
        
        #** Gestion des sequences par defaut ***
        if ( $FlagDef == 1 ) { ReadDef($KeyFile); }
        
        #** Gestion des arbres ps ***
        my %hPS;       # hachage { fichier ps -> [ header, leaf, footer ] }
        my $FicPs;
        $fColor = 0;
        if ( $FlagColor == 1 ) {
            if ( !opendir(REP, $FileRep) ) { AffError("Can't open directory \'$FileRep\' !"); }
            my @T1 = grep m/$KeyFile.*\.ps$/i, readdir REP;   # liste tous les fichiers $KeyFile*.ps sans tenir compte de la case
            closedir REP;
            for $FicPs ( @T1 ) {
                if ( $FicPs =~ m/$KeyFile\.ps/ or $FicPs =~ m/$KeyFile\_.*\.ps/ ) {
                    if ( open(FIC, $FileRep.$SepDir.$FicPs) ) {
                        $bHeader = "";
                        $bFooter = "";
                        $bLeafIn = "";
                        $bLeafOut = "";
                        # lecture du fichier
                        $L = join("",  <FIC>);   # concatenation de toutes les lignes en une liste               
                        close FIC;
                        if ( $L =~ m/(.+)(%Impression des BP.+)/s ) {
                            $bFooter = $2;
                            $L = index($1, "(");
                            $bHeader = substr($1,0,$L-1);
                            $bHeader .= "\n0.5 0.5 0.5 setrgbcolor\n";     # ecrit feuilles par defaut en gris
                            $bLeafIn = substr($1,$L);
                            }
                        if ( length($bLeafIn) > 0 and length($bLeafIn) > 0 ) {
                            # au moins 1 fichier *.ps correct
                            $hPS{$FicPs} = [$bHeader, $bLeafIn, $bFooter];
                            $fColor = 1;
                            }
                        }
                    }
                }
            }
            
        #** Recherche la meilleure sequence de chaque groupe ***
        $hStatFic{$F}[0] = 0;
        $hStatFic{$F}[1] = 0;
        $hStatFic{$F}[2] = 0;
        $lStatFicTmp[0] = 0;                    # stat globales pour le fichier
        $lStatFicTmp[1] = 0;                    # 0->NbMisPos   1->NbMisOTU    2-> Chim
        $lStatFicTmp[2] = 0;
        for $Grp ( @Grp ) {
            $hConcatTmp{$Grp} = "";             # selection pour chaque OTU 
            @{$hStatGrpTmp{$Grp}}[0] = 0;       # stat pour l'OTU dans le fichier:
            @{$hStatGrpTmp{$Grp}}[1] = 0;       # 0->NbMisPos   1->NbMisOTU    2-> Chim
            @{$hStatGrpTmp{$Grp}}[2] = 0;
            #** Parmi les sequences completes ou quasi ***
            if ( exists($hSeqFile{$KeyFile}{$Grp}) ) {
                #-- Groupe absent du fichier ---
                if ( @{$hSeqFile{$KeyFile}{$Grp}}[2] == 0 ) {
                    $Name = "";
                    $bLOG4 .= "$Grp : no sequence for this OTU\n"
                    }
                #-- Traite les sequences completes ---
                else {
                    ($Seq, $Name, $L) = SearchBestSeq($F, $Grp, $LgFile);
                    $bLOG2 .= $L;
                    if ( $Name eq "" ) {
                        if ( $FlagRemoveAll == 0 )
                            { $bLOG4 .= "$Grp : no sequence with correct distance\n"; }
                        else {
                            # suppression de tout le fichier
                            $bRemFile .= "$F  ";
                            $bLOG4 = "\n=== File: $F ===\nremoved file\n";
                            print LOG4 $bLOG4;
                            $iSeq = 0;
                            $NbGrpAbs = $#Grp+1;
                            for ( @Grp ) { 
                                @{$hStatGrp{$_}}[0] += $LgFile;
                                @{$hStatGrp{$_}}[1]++;
                                @{$hStatGrp{$_}}[2] = 0;
                                $hGrpFic{$F}{$_} += $LgFile;
                                }
                            @{$hStatFic{$F}}[0] = $NbGrpAbs*$LgFile;
                            @{$hStatFic{$F}}[1] = $NbGrpAbs;
                            @{$hStatFic{$F}}[2] = 0;
                            if ( length($bLOG2) > 0 and $FlagNoPuz == 0 ) {
                                $L = "\n\n=== $F ===\n\n";
                                $L .= pack("A40A12A10", "OTU", "Max in-OTU", "Threshold") . "\n";
                                $L .= pack("A2A52A20", '  ', "Sequence", "out-OTU Dist. Ave.") . "\n";
                                $L .= ("-" x 70). "\n";
                                print LOG2 $L . $bLOG2 . "\n -> removed file\n";
                                }
                            goto REMOVEALL;
                            }
                        }
                    #-- Gestion des arbres ps ---
                    elsif ( $fColor == 1 ) {
                        for $FicPs ( keys %hPS ) {
                            $bLeafIn = $hPS{$FicPs}[1];
                            $bLeafIn = WriteColor($bLeafIn, $Name, "0 0 1 setrgbcolor", "");   # bleu
                            $hPS{$FicPs}[1] = $bLeafIn;
                            }
                        }
                    }
                }
            
            #** Resultante de chimeres ***
            elsif ( exists($hSeqChim{$KeyFile}{$Grp}) ) {
                $Seq = $hSeqChim{$KeyFile}{$Grp};
                ($flag, $n) = TooShort($MaxBlanc, $Seq);
                if ( $flag == 1 ) {
                    $Name = "";
                    $bLOG4 .= "$Grp : chimera is too short\n";
                    }
                else {
                    $Name = $hNameChim{$KeyFile}{$Grp};
                    if ( $Name =~ m/chimera/ ) {
                        @{$hStatGrpTmp{$Grp}}[2]++;
                        $lStatFicTmp[2]++;
                        }
                    #-- Gestion des arbres ps ---
                    if ( $fColor == 1 ) {
                        $n = $hNameChim{$KeyFile}{$Grp};
                        while ( $n =~ m/((\w|\d|@|\s|\._|\.|-|_)*) \(\d*\),/g ) {
                            $L = $1;
                            $L =~ s/^\s+//;
                            $L =~ s/\s+$//;
                            for $FicPs ( keys %hPS ) {
                                $bLeafIn = $hPS{$FicPs}[1];
                                $bLeafIn = WriteColor($bLeafIn, $L, "0 0.5 0 setrgbcolor");    # vert
                                $hPS{$FicPs}[1] = $bLeafIn;
                                }
                            }
                        }   
                    }
                }
            
            #** Groupe absent du fichier ***
            else {
                $Name = "";
                $bLOG4 .="$Grp : no sequence for this OTU\n";
                }

            # sauvegarde sequence pour le groupe
            if ( $Name ne "" ) {
                # ecrit .extali.*ali des sequences retenues
                $bEXT .= ">$Grp\n$Seq\n";
                $Name =~ s/\|//;
                $bLOG4 .= "$Grp : $Name\n";
                $iSeq++;
                # compte les trous
                $n=0;
                $n++ while ( $Seq =~ m/\?|\*|\.|\$|\-|X|\s/g );
                @{$hStatGrpTmp{$Grp}}[0] = $n;
                $lStatFicTmp[0] += $n;
                }
            else {
                # pas de sequence pour ce groupe dans le fichier initial
                $NbGrpAbs++;
                @{$hStatGrpTmp{$Grp}}[1]++;
                $lStatFicTmp[1]++;
                $Seq = "?" x $LgFile;
                @{$hStatGrpTmp{$Grp}}[0] = $LgFile;
                $lStatFicTmp[0] += $LgFile;
                }
            $hConcatTmp{$Grp} = $Seq;
            }   # boucle sur les OTUs
            
        #** ecriture logs **************************************
        if ( length($bLOG2) > 0 and $FlagNoPuz == 0 ) {
            $L = "\n\n=== $F ===\n\n";
            $L .= pack("A40A12A10", "OTU", "Max in-OTU", "Threshold") . "\n";
            $L .= pack("A2A52A20", '  ', "Sequence", "out-OTU Dist. Ave.") . "\n";
            $L .= ("-" x 70). "\n";
            print LOG2 $L . $bLOG2;
            }
        if ( !open(EXT, "> $MisGenRep$F") )
            { AffError("Can't open for writing file \'$MisGenRep$F\' !"); }
        print EXT $bEXT;
        close EXT;
        print LOG4 $bLOG4;

        if ( $FlagDef != -1 ) { CloseDef; }
        
        #** sauvegarde longueur et nombre de sequences pour fichier **
        if ( $iSeq > 0 )
#            { print LEN "$F\t".$LgFile."\t$iSeq\n"; }
            { 
            print LEN "$F\t$Deb\t".($Deb+$LgFile-1)."\t[$LgFile\t$iSeq]\n";
            $Deb += $LgFile;
            }
            
        #** Gestion des statistiques et concatenation **********
        @{$hStatFic{$F}}[0] += $lStatFicTmp[0];
        @{$hStatFic{$F}}[1] += $lStatFicTmp[1];
        @{$hStatFic{$F}}[2] += $lStatFicTmp[2];
        
        for ( @Grp ) {
            $hGrpFic{$F}{$_} = @{$hStatGrpTmp{$_}}[0];
            @{$hStatGrp{$_}}[0] += @{$hStatGrpTmp{$_}}[0];
            @{$hStatGrp{$_}}[1] += @{$hStatGrpTmp{$_}}[1];
            @{$hStatGrp{$_}}[2] += @{$hStatGrpTmp{$_}}[2];
            $hConcat{$_} .= $hConcatTmp{$_};
            }
        if ( $NbGrpAbs <= $NbMisGrp )
            { WriteMissing($F, $NbGrpAbs, $LgFile); }
            
        #** Gestion des arbres ps ***
        if ( $fColor == 1 ) {
            for $FicPs ( keys %hPS ) {
                if ( !open(COLOR, "> $CefRep$FicPs") ) 
                    { AffError("Can't open for writing the file \'$CefRep$FicPs\' !"); }
                else {
                    # ecriture du fichier
                    print COLOR $hPS{$FicPs}[0].$hPS{$FicPs}[1]."0 0 0 setrgbcolor\n".$hPS{$FicPs}[2];    # ecrit BP en noir
                    close COLOR;
                    }
                }
            }
REMOVEALL:

        }   # boucle sur les fichiers
        
    print LOG4 "\n";
    close LOG4;
    close LEN;

    #** statistiques pour l'ensemble de la concatenation **
    WriteStat();

    if ( $FlagNoPuz == 0 ) {        
        print LOG2 "\n";
        close LOG2;
        }
    
    #-- statistiques sur les groupes absents -------------------
    StatMissingGrp($bRemFile);
    return %hConcat;
}

        
#### Modification couleurs des feuilles dans un arbre PS #######
sub WriteColor {
    my ($bIn, $Name, $bColor) = @_;
    
    #** une sequence choisie **
    if ( $bIn =~ m/(.+)$Name(.*)/s ) {
        $L = rindex($1, "(");
        if ( $L == 0 ) { $bIn = "";}
        else { $bIn = substr($1, 0, $L-1); }
        $bIn .= "\n$bColor\n( ";                     # ecrit nom de sequence en bleu ou vert
        $bIn .= $Name;
        $bIn .= ") show\n0.5 0.5 0.5 setrgbcolor";   # ecrit en gris par defaut
        $L = index($2, ") show");
        $bIn .= substr($2, $L+length(") show"));
        }
    return $bIn;
}    


### Recherche la meilleure sequence ############################
sub SearchBestSeq {
    my ($FileAli, $Grp, $LgrMax) = @_;
    my $KeyFile = "";
    my @ListSeq = ();        #liste des sequences de l'OTU
    my $Seq = "?" x $LgrMax;
    my $Id = "";
    my $flag;
    my $Name = "";
    my $buf;
    my %SeqRetained;         # sequences retenues = +longues ou -divergentes
    my $nbSeq = 0;           # nb de sequences retenues
    my $Index;               # index sequence la +longue ou -divergente dans la liste des sequences retenues
    my $SpecName;

    $FileAli =~ m/(.*)\.ali$/i;
    $KeyFile = $1;
    @ListSeq = split("\n", @{$hSeqFile{$KeyFile}{$Grp}}[1]);

    #** une seule sequence : on la prend ***********************
    if ( @{$hSeqFile{$KeyFile}{$Grp}}[2] == 1 ) {
        if ( $ListSeq[0] =~ m/^\d+\s+(.*)/ )
            { return($1, @{$hSeqFile{$KeyFile}{$Grp}}[3], ''); }
        }
        
    #** prend la sequence la plus longue ***********************
    elsif ( $FlagNoPuz == 1 ) {
        my @lName;
        $L = @{$hSeqFile{$KeyFile}{$Grp}}[3];
        @lName = split(/\|/,$L);
        my $nbMin = 100000;
        my $n;
        my $iName = 0;
        for ( @ListSeq) {
            if ( $_ =~ m/^(\d+)\s+(.*)/ ) {
                $Id = $1;
                $Seq = $2;
                ($flag, $n) = TooShort($LgrMax, $Seq);
                if ( $n < $nbMin ) {
                    $nbMin = $n;
                    # reinitialise la liste des sequences potentielles
                    %SeqRetained = ();
                    $nbSeq = 0;
                    goto ADDSEQ_LG; 
                    }
                elsif ( $n == $nbMin ) {
ADDSEQ_LG:
                    # ajoute la sequence
                    $SeqRetained{$Id}[0] = $Seq;
                    $SeqRetained{$Id}[1] = $lName[$iName];
                    $nbSeq++;
                    }
                $iName++;
                }
            }
        # aucune sequence retenues
        if ( $nbSeq == 0 ) 
            { return("?" x $LgrMax, "", ''); }
        # selectionne la premiere sequence dans l'ordre du .OTU
        else {
            $Index = 1000;
            goto IDENT;
            }
        }
        
    #** verifie si une et une seule sequence dans .DEF *********
    if ( $FlagDef == 1 ) {
        if ( length($lSeqDef) > 5 ) {       # .DEF existe
            ($flag, $Seq, $Name) = SearchOneSeqDef($KeyFile, $Grp);
            #** prend choix user si celui si est unique
            if ( $flag == 1 ) {
                if ( $Name eq "" ) { $Seq = "?" x $LgrMax; }
                return($Seq, $Name, '');
                }
            #** pas de sequence
            if ( $flag == -1 )
                { return("?" x $LgrMax, '', ''); }
            }
        else
            { 
            $flag = 0; }
        }
    #** Calcule automatique par CEF
    else
        { $flag = -1; }

    #** Calcul toujours automatique
    if ( $FlagDefAuto == 1 ) { $flag = -1; }
    
    #* on cherche la meilleure sequence du groupe **************
    my $mean = 0;
    my $nbIn = 0;
    my $min = 1000;
    my $max = 0;
    my ($SeqName, $L1, $L2) = ("", "", "");
    my $S;
    my $SeqAve;
#!# drapeau pour test de SCaFoS:
    $FlagTest = '';      # utilisation normale avec parametres de l'utilisateur
#!#    $FlagTest = 'min';   # selection des sequences les moins divergentes
#!#    $FlagTest = 'max';   # selection des sequences les plus divergentes
#!#    $FlagTest = 'rand';  # selection aleatoire d'une sequence

    #-- determination aleatoire ---
    if ( $FlagTest eq 'rand' ) {
        $Id = int(rand scalar(@ListSeq));
        $S = $ListSeq[$Id];
        if ( $S =~ m/^(\d+)\s+(.*)/ ) {
            $Id = $1;
            $Seq = $2;
            $SeqName = SearchGrpSpe($Id, $KeyFile, 1);
            }
        if ( $SeqName ne "" ) { return($Seq, $SeqName); }
        else { return("?" x $LgrMax, '', ''); }
        }
    #-- determination selon la distance hors groupe ---
    else {
        for $S ( @ListSeq ) {
            if ( !($S =~ m/^(\d+)\s+(.*)/) ) { next; }
            $Id = $1;
            $Seq = $2;
            $SeqName = SearchGrpSpe($Id, $KeyFile, 1);
            if ( $SeqName eq "" ) { next; }
            $SeqAve = @{$hCorres{$KeyFile}{$SeqName}}[2];
            # recherche sequence a la distance hors groupe la plus grande
            if ( $FlagTest eq 'max' ) {
                if ( $SeqAve > $max ) {
                    # reinitialise la liste des sequences potentielles
                    $max = $SeqAve;
                    %SeqRetained = ();
                    $nbSeq = 0;
                    goto ADDSEQ; 
                    }
                elsif ( $SeqAve == $max ) {
                    goto ADDSEQ; 
                    }
                }
            # recherche sequence a la distance hors groupe la plus petite
            else {
                if ( $SeqAve < $min ) {
                    # reinitialise la liste des sequences potentielles
                    $min = $SeqAve;
                    %SeqRetained = ();
                    $nbSeq = 0;
                    goto ADDSEQ; 
                    }
                elsif ( $SeqAve == $min ) {
ADDSEQ:
                    # ajoute la sequence
                    $nbSeq++;
                    $SeqRetained{$Id}[0] = $Seq;
                    $SeqRetained{$Id}[1] = $SeqName;
                    }
                $mean += $SeqAve;
                $nbIn++;
                }
            }
        $mean /= $nbIn;
        }

    #-- verifie si groupe hors normes ---
    my $nb;
    my $bLOG2Tmp = '';
    if ( $FlagTest eq '' ) {
        $nb = 0;
        if ( @{$hSeqFile{$KeyFile}{$Grp}}[4] > ($PctAve * $mean) ) {
            # comportement automatique : elimine le groupe
            if ( $flag == -1 ) {
                $bLOG2Tmp .= pack("A40A12A10",$Grp, @{$hSeqFile{$KeyFile}{$Grp}}[4], $PctAve * $mean) . "\n";
                for $Id ( @{$hSeqFile{$KeyFile}{$Grp}[5]} ) {
                    for $S ( keys %{$hCorres{$KeyFile}} ) {
                        if ( $Id == @{$hCorres{$KeyFile}{$S}}[0] ) {
                            $L = pack("A2A52A10", '  ',  $S, @{$hCorres{$KeyFile}{$S}}[2]) . "\n";
                            $bLOG2Tmp .= $L;
                            next;
                            }
                        }
                    }            
                return("?" x $LgrMax, '', $bLOG2Tmp);
                }
            # interrogation de l'utilisateur
            else {
                if ( @{$hSeqFile{$KeyFile}{$Grp}}[4] == 0 )
                    #Choix par SCaFoS si toutes les sequences sont identiques
                    { $flag = -1; }
                else {
                    $buf =  " SCaFoS has elliminated all sequences of the OTU \'$Grp\' because\n";
                    $buf .= " maximal ingroup distance is higher than " . 100*$PctAve. "\% of outgroup distance average\n";
                    ($nb, $Seq, $Name) = LetUserDef($buf, $KeyFile, $Grp, 'red');

                    # elimine le groupe
                    if ( $nb == 0 ) {
                        $bLOG2Tmp .= pack("A40A12A10",$Grp, @{$hSeqFile{$KeyFile}{$Grp}}[4], $PctAve * $mean) . "\n";
                        for $Id ( @{$hSeqFile{$KeyFile}{$Grp}[5]} ) {
                            for $S ( keys %{$hCorres{$KeyFile}} ) {
                                if ( $Id == @{$hCorres{$KeyFile}{$S}}[0] ) {
                                    $L = pack("A2A52A10", '  ',  $S, @{$hCorres{$KeyFile}{$S}}[2]) . "\n";
                                    $bLOG2Tmp .= $L;
                                    next;
                                    }
                                }
                            }            
                        return("?" x $LgrMax, "", $bLOG2Tmp);
                        }
                    # retourne sequence choisie par user
                    else
                        { return($Seq, $Name, $bLOG2Tmp); }
                    }
                }
            }
        }
    #-- selection toujours automatique ---
    else
        { $flag = -1; }

    #-- selection automatique ----------------------------------
    if ( $flag == -1 ) {
AUTO:
        # recupere sequence a l'index le plus faible si plusieurs sequences ont la distance la plus faible
        if ( $nbSeq > 1 ) {
            $Index = 1000;
IDENT: 
            for $Id (keys %SeqRetained) {
                for $S ( keys %{$hCorres{$KeyFile}} ) {
                    if ( $Id == @{$hCorres{$KeyFile}{$S}}[0] ) {
                        $SpecName = SearchSpecName($S, "@");
                        if ( length($SpecName) > 0 ) {
                            if ( $hGrpSpe{$Grp}{$SpecName} < $Index ) {
                                $Seq = $SeqRetained{$Id}[0];
                                $Name = @{$SeqRetained{$Id}}[1];
                                $Index = $hGrpSpe{$Grp}{$SpecName};
                                }
                            next IDENT;
                            }
                        }
                    }
                }
            }       
        # recupere la sequence unique
        elsif ( $nbSeq == 1 ) {
            for ( keys %SeqRetained ) {
                $Seq = $SeqRetained{$_}[0];
                $Name = @{$SeqRetained{$_}}[1];
                }
            }
        }
                
    #-- selection manuelle par utilisateur ---------------------
    else {
        if ( @{$hSeqFile{$KeyFile}{$Grp}}[4] == 0 )
            #Choix par SCaFoS si toutes les sequences sont identiques
            { $flag = -1; goto AUTO;}
        else {
            $buf = " Many sequences in the OTU \'$Grp\' : you could specify a default sequence\n";
            ($nb, $Seq, $Name) = LetUserDef($buf, $KeyFile, $Grp, "#003F87");

            # pas de selection par l'utilisateur
            if ( $nb < 1 ) {
                $Seq = "";
                $Name = "";
                if ( $nb == 0 ) { goto AUTO; }
                }
            }
        }
        
    # c'est fini : on a peut-etre une sequence !!!!!!!!
    return($Seq, $Name, $bLOG2Tmp);
}

1;

### fin du fichier                                           ###
################################################################
